
import java.util.Scanner;
import it.sauronsoftware.ftp4j.FTPClient;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Console app to perform simple FTP client operations.
 */
public class ConsoleFtpApp {

    private static Scanner in;
    private static FTPBuilder m_Builder = null;
    private static FTPClient m_Client = null;
    private static SingleThreadedTransferEngine m_Engine;

    /**
     * 
     * @param args No command line arguments are expected or processed
     */
    public static void main(String[] args) {

        String cmd = "";
        in = new Scanner(System.in);
        m_Engine = new SingleThreadedTransferEngine();

        // For testing/debugging, a local FileZilla ftp server was used
        // Using FileZilla's speed limiting allows us to easily simulate 
        // long transfer times for large files 
        // Hard-coding the credentials here saves time
        m_Builder = new FTPBuilder();
        m_Builder.setUsername("cmsc335");
        m_Builder.setPassword("spring2015");
        m_Builder.setHost("54.86.86.246");
        m_Builder.setPort(21);
        try {
            m_Client = m_Builder.buildClient();
        } catch (IOException ex) {
            Logger.getLogger(ConsoleFtpApp.class.getName()).log(Level.SEVERE, null, ex);
        }
        if (m_Client.isConnected()) {
            System.out.println("Connected.");
            m_Engine.start();
        }

        while (!cmd.equals("exit")) {
            System.out.print("ftp>");
            cmd = in.next();
            switch (cmd) {
                case "connect": {
                    m_Builder = exec_Connect();
                }
                break;
                case "put":
                    exec_Put();
                    break;
                case "get":
                    exec_Get();
                    break;
                case "jobs": {
                    exec_Jobs();
                }
                break;
                case "help":
                case "?":
                    exec_Help();
                    break;
                case "exit":
                    exec_Exit();
                    break;
            }
        }
    }

    /**
     * Executes the "Connect" command 
     * Constructs an FTPBuilder and uses it to create an FTPClient
     * for use in the UI as needed.
     * 
     * @return The FTPBuilder object to construct other FTP connections later on
     */
    private static FTPBuilder exec_Connect() {
        String user, pass, host;
        int port;
        FTPBuilder builder;

        user = in.next();
        pass = in.next();
        host = in.next();
        port = in.nextInt();
        builder = new FTPBuilder();
        builder.setUsername(user);
        builder.setPassword(pass);
        builder.setHost(host);
        builder.setPort(port);
        try {
            m_Client = builder.buildClient();
            if (m_Client.isConnected()) {
                System.out.println("Connected.");
                m_Engine.start();
            }
        } catch (Exception ex) {
            System.out.printf("Failed to open connection: %s\n", ex.getMessage(), m_Client);
            return null;
        }

        return builder;
    }

    private static void exec_Put() {
        String filename;
        filename = in.nextLine().trim();
        m_Engine.addJob(new PutJob(m_Builder, filename));
        System.out.println("Job added.");
    }

    private static void exec_Get() {
        String filename;
        filename = in.nextLine().trim();
        m_Engine.addJob(new GetJob(m_Builder, filename));
        System.out.println("Job added.");
    }

    private static void exec_Jobs() {
        for (TransferJob job : m_Engine.getJobs()) {
            try {
                System.out.printf("Transfer %s -> %s...%s\n",
                        job.getSrcResource(), job.getDestResource(), job.getState());
            } catch (MalformedURLException ex) {
                System.out.printf("Exception obtaining job information: %s\n", ex.getMessage());

            }
        }
    }

    private static void exec_Help() {
        System.out.println("Commands:");
        System.out.println("\tconnect <username> <password> <host> <port>");
        System.out.println("\tput <filename>");
        System.out.println("\tget <filename>");
        System.out.println("\tjobs");
        System.out.println("\thelp");
        System.out.println("\texit");

    }

    private static void exec_Exit() {
        if (null != m_Client) {
            if (m_Client.isConnected()) {
                try {
                    m_Client.disconnect(true);
                } catch (Exception ex) {
                    Logger.getLogger(ConsoleFtpApp.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
    }

}
