
import java.net.MalformedURLException;
import java.util.Collection;
import javax.swing.table.AbstractTableModel;

/**
 * An implementation of AbstractTableModel to represent transfer job information 
 * 
 */
public class JobTableModel extends AbstractTableModel{
    /*
    * Keep List of TransferJob
    */
    private final Collection<TransferJob> jobs;

    /**
     * The default constructor
     * @param jobs List of TransferJob
     */
    public JobTableModel(Collection<TransferJob> jobs) {
        this.jobs = jobs;
    }

    /*
    * Returns row count of table
    */
    @Override
    public int getRowCount() {
        return jobs.size();
    }

    /*
    * Returns column count of table
    */
    @Override
    public int getColumnCount() {
        return 4;
    }

    /*
    * Returns column name
    */
    @Override
    public String getColumnName(int column) {
        switch (column) {
            case 0:
                return "Source";
            case 1:
                return "Destination";
            case 2:
                return "Progress";
            case 3:
                return "Error";
            default:
                throw new AssertionError();
        }
    }

    /*
    * Returns the value at position rowindex and columnindex
    */
    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
        try {         
            TransferJob job = (TransferJob) jobs.toArray()[rowIndex];
            switch (columnIndex) {
                case 0:
                    return job.getSrcResource();
                case 1:
                    return job.getDestResource();
                case 2:
                    return String.format("%.1f",job.getPercentComplete()*100);
                case 3:
                    return job.getErrorMessage();
                default:
                    throw new AssertionError();
            }
        } catch (MalformedURLException ex) {
            return null;
        }
    }
    
}
