


import java.util.Collection;

/**
 * Maintains a collection of jobs and initiates a transfer on each job added to
 * the engine. Different engine implementations may process the jobs in a
 * different order or concurrently.
 *
 * @author mpilone
 */
public interface TransferEngine {

  /**
   * Returns an unmodifiable collection of the jobs that were added to the
   * engine in the order that they were added. The jobs may be in different
   * states including transfered, transferring, error, or pending.
   *
   * @return the collection of jobs
   */
  public Collection<TransferJob> getJobs();

  /**
   * Adds a job to the engine. Depending on the job engine implementation, the
   * job may start immediately or be queued for future execution. Jobs added to
   * the engine must be in the {@link JobState#READY} state.
   *
   * @param job the job to add
   */
  public void addJob(TransferJob job);

  /**
   * Starts the engine. This method is normally called before adding any jobs to
   * the engine but it may be called afterward. Starting a engine that is
   * already started does nothing.
   */
  public void start();

  /**
   * Stops the engine. This method will block (that is, not return) until the
   * engine has completed all transferring jobs. Stopping an engine that is
   * already stopped does nothing.
   */
  public void stop();

}
